;;; debug-common.scm

;; Copyright (C) 2002, 2003, 2005, 2007 Thien-Thi Nguyen
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Author: Thien-Thi Nguyen <ttn@gnu.org>

;;; Commentary:

;; This module contains debugging support for the (lang foo-debug) modules.

;;; Code:

(define-module (lang debug-common)
  #:autoload (lang grammar) (production-symbol production-expansion)
  #:export (;; pretty printers
            pp-lexer-spec
            pp-grammar
            ;; nicknames
            nick
            nick!))

;; Pretty-print the lexer-spec @var{spec}.
;;
(define (pp-lexer-spec spec)
  (for-each (lambda (item)
	      (display (car item))
	      (display #\tab) (display #\tab)
	      (cond
	       ((symbol? (cadr item))	(display (cadr item)))
	       ((not (cadr item))	(display "ignored"))
	       (else			(display "!!! special")))
	      (display #\space)
	      (if (memq #:shortest item)
		  (display #:shortest))
	      (newline))
	    spec))

;; Pretty-print the grammar @var{spec}.
;;
(define (pp-grammar spec)
  (let loop ((current-sym #f)
	     (pos spec))
    (or (null? pos)
        (let ((prod (car pos)))
          (if (eq? current-sym (production-symbol prod))
              (begin
                (display (make-string (string-length current-sym) #\space))
                (display "  | "))
              (begin
                (newline)
                (display (production-symbol prod))
                (display " -> ")))
          (for-each (lambda (elt) (display elt) (display " "))
                    (production-expansion prod))
          (cond ((--nick prod)
                 => (lambda (nick)
                      (display #\tab) (display #\tab)
                      (display nick))))
          (newline)
          (loop (production-symbol prod) (cdr pos))))))


(define --nick (make-object-property))

;; Return the nickname of @var{obj} if available.
;; If @var{obj} is @code{#f} or the empty list @code{()}, return @var{obj}.
;; Otherwise, return the memory address of @var{obj} as a hex string.
;;
(define (nick obj)
  (and obj (or (--nick obj)
               (and (list? obj)
                    (null? obj)
                    obj)
               (number->string (object-address obj) 16))))

;; Set the nickname for @var{obj} to @var{nick}.
;; Return @var{obj}.
;;
(define (nick! obj nick)
  (set! (--nick obj) nick)
  obj)

;;; debug-common.scm ends here
