;;; lr0-debug.scm

;; Copyright (C) 2002, 2003, 2004, 2007 Thien-Thi Nguyen
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Author: Thien-Thi Nguyen <ttn@gnu.org>

;;; Commentary:

;; In lr0.scm there is a comment:
;;
;;   Some day, grammar debugging tools will be added to this library.
;;
;; Well, that day is here!  This module is a drop-in replacement for
;; module (lang lr0) with some additional debugging features.

;;; Code:

(define-module (lang lr0-debug)
  #:use-module ((ice-9 gumm) #:select (fob-ref fob-define! module-name->fob))
  #:use-module (ice-9 common-list)
  #:use-module (lang debug-common)
  #:use-module (lang grammar)
  #:use-module ((lang lr0) #:prefix ACTUAL:)
  #:export (;; mimic exports from module (lang lr0)
            item-set-start-kernel
            item-set-successor-kernel
            item-set-reductions
            item-set-closure
            grammar-item-table
            ;; "native" (defined here)
            VALIDATE:production
            VALIDATE:item
            VALIDATE:item-set
            grammar-with-nicknames
            display-grammar-item-table
            display-item-set
            display-item-set-closure))

;; intimacy w/ module (lang lr0)

(define *lr0* (module-name->fob '(lang lr0)))

(defmacro intimate-lr0 (name)
  `(define ,name (fob-ref *lr0* ',name)))

(intimate-lr0 item-production)
(intimate-lr0 item-position)
(intimate-lr0 item-index)
(intimate-lr0 item-set)

;; type invariance validation

;; Check production-specific invariants for @var{obj}.
;; If there is a problem, signal error with message of the form:
;; @smallexample
;; supposed production INVARIANT-ERROR
;; @end smallexample
;; where @var{invariant-error} is one of:
;; @itemize
;; @item not a list
;; @item has length not >= 2
;; @item car not a symbol
;; @item cadr not a list of symbols
;; @end itemize
;;
(define (VALIDATE:production obj)
  (or (list? obj)
      (error "supposed production not a list:" obj))
  (or (< 2 (length obj))
      (error "supposed production has length not >= 2:" obj))
  (or (symbol? (car obj))
      (error "supposed production car not a symbol:" (car obj)))
  (or (every symbol? (cadr obj))
      (error "supposed production cadr not a list of symbols:" (cadr obj))))

;; Check item-specific variants for @var{obj}.
;; If there is a problem, signal error with message of the form:
;; @smallexample
;; supposed item INVARIANT-ERROR
;; @end smallexample
;; where @var{invariant-error} is one of:
;; @itemize
;; @item not a list
;; @item cdr CDR not a subset of the expansion EXPANSION
;; @end itemize
;;
(define (VALIDATE:item obj)
  (or (list? obj)
      (error "supposed item not a list:" obj))
  ;; car is the production: (SYMBOL (SYM SYM SYM) . REDUX)
  (VALIDATE:production (car obj))
  ;; cdr is some subset of the production expansion
  (let ((expansion (cadr (car obj))))
    (or (some (lambda (n)
                (eq? (list-tail expansion n)
                     (cdr obj)))
              (iota (1+ (length expansion))))
        (error "supposed item cdr:" (cdr obj) "not a subset of the expansion:"
               expansion)))
  obj)

;; Check item-set-specific variants for @var{obj}.
;; If there is a problem, signal error with message of the form:
;; @smallexample
;; supposed item-set INVARIANT-ERROR
;; @end smallexample
;; where @var{invariant-error} is one of:
;; @itemize
;; @item not a list
;; @end itemize
;; Additionally, check each member of the item-set with
;; @code{VALIDATE:item}.
;;
(define (VALIDATE:item-set obj)
  (or (list? obj)
      (error "supposed item-set not a list: " obj))
  (for-each VALIDATE:item obj)
  obj)

;; hashing for display

;; Decorate @var{grammar} productions with nicknames and return it.
;; For each production, call @code{VALIDATE:production} on it and
;; then decorate with a string of the form "PRODUCTION-SYMBOL[ADDRESS]".
;;
(define (grammar-with-nicknames grammar)
  (let ((ht (make-hash-table #:size 31)))
    (for-each (lambda (p)
                (VALIDATE:production p)
                (let ((sym (production-symbol p)))
                  (nick! p (simple-format
                            #f "~A[~A]"
                            sym
                            (hashq-set! ht sym
                                        (1+ (or (hashq-ref ht sym)
                                                -1)))))))
              grammar))
  grammar)

;; display

;; Display item @var{item} to the current output port.
;;
(define (display-item item)
  (simple-format #t "~A: ~A | ~A\n"
                 (item-index item)
                 (nick (item-production item))
                 (item-position item)))

;; Display grammar item table @var{table} to the current output port.
;;
(define (display-grammar-item-table table)
  (do ((i 0 (1+ i)))
      ((= i (vector-length table)))
    (display-item (vector-ref table i))))

;; Display the item-set @var{is} to the current output port.
;;
(define (display-item-set is)
  (for-each display-item is))

;; Display the item-set closure @var{isc} to the current output port.
;;
(define (display-item-set-closure isc)
  (for-each display-item isc))

;; poke debugging vars

(fob-define! *lr0* 'item-closure-debug-pk
             (lambda (item caar-stack actual-x)
               (display-item item)
               (simple-format #t "caar-stack: ")
               (display-item caar-stack)
               (for-each (lambda (item)
                           (simple-format #t "--actual-x: ")
                           (display-item item))
                         actual-x)))

;; wrap the actual

(define item-set-start-kernel     ACTUAL:item-set-start-kernel)
(define item-set-successor-kernel ACTUAL:item-set-successor-kernel)
(define item-set-reductions       ACTUAL:item-set-reductions)
(define item-set-closure          ACTUAL:item-set-closure)
(define grammar-item-table        ACTUAL:grammar-item-table)

;;; lr0-debug.scm ends here
