;;; yy.scm

;; Copyright (C) 2002, 2003, 2004, 2007 Thien-Thi Nguyen
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Commentary:

;; This module abstracts module `(lang lr1)' in three ways:
;; - The grammar must not handle <eoi> tokens directly.
;; - The grammar cannot use symbol `%%YY' (anywhere).
;; - The grammar is expanded to handle YYACCEPT, YYABORT and YYERROR calls.

;;; Code:

(define-module (lang yy)
  #:use-module (lang grammar)
  #:use-module (lang lex)
  #:use-module (lang lr1-debug)
  #:export (yy-parse))

(for-each (lambda (msg)
            (display msg (current-error-port)))
          (list
           "WARNING: Module (lang yy) incomplete!\n"
           "(Feel free to hack on it and post patches, however.)\n"))


;;; Support

(define (splice-YY tree)
  (if (list? tree)
      (let ((head (car tree)))
        (cond ((and (symbol? head) (eq? 'YYERROR head))
               '(throw '%%YY 'error #t))
              ((and (symbol? head) (eq? 'YYABORT head))
               '(throw '%%YY 'abort #t))
              (else
               (append (if (and (symbol? head)
                                (eq? 'YYACCEPT head))
                           '(throw '%%YY 'accept)
                           (list (splice-YY head)))
                       (map splice-YY (cdr tree))))))
      tree))

(define (prep-grammar grammar)
  (map (lambda (p)
         (apply list
                (production-symbol p)
                (production-expansion p)
                (splice-YY (production-body p))))
       (append '()
               ;;`(($start$ (,(production-symbol (car grammar)) $eoi-shifter$)
               ;;           (YYACCEPT $1))
               ;;  ($eoi-shifter$ ($eoi-shifter$ <eoi>) #f)
               ;;  ($eoi-shifter$ () #f))
               grammar)))

(define (absorb-soft-eoi lexer)
  (letrec ((err (lambda (cont states values token lexer)
                  (cond
                   ((and (eof-token? token) (not (cadr token)))
                    (write-line "error handler continuing")
                    (cont states values (lexer) lexer err))
                   (else
                    (error "unexpected:" token))))))
    err))

;; Parse @var{grammar} with @var{lexer} proc in @var{module}'s environment.
;; NOTE: This procedure is still highly experimental; use it only if you're
;; ready to go look at the source to figure out and/or fix problems.
;;
(define (yy-parse grammar lexer module)
  (catch '%%YY
         (lambda ()
           (lr1-parse (prep-grammar grammar)
                      lexer
                      (absorb-soft-eoi lexer)
                      module))
         (lambda args
           (case (cadr args)
             ((accept) (if (= 3 (length args))
                           (caddr args)
                           (cddr args)))
             ((abort) #f)
             (else (error "unexpected %%YY:" (cdr args)))))))

;;; yy.scm ends here
