;;; srfi-13.scm --- String Library

;; Copyright (C) 2002, 2003, 2004, 2005, 2007 Thien-Thi Nguyen
;; Copyright (C) 2001, 2002 Free Software Foundation, Inc.
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 3, or
;; (at your option) any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Commentary:

;; This module is fully documented in the Guile Reference Manual.

;;; Code:

(define-module (srfi srfi-13)
  #:use-module (srfi srfi-13-14-sup)
  #:re-export (
               ;; Predicates
               ;; string? string-null?       <= in the core
               string-any
               string-every

               ;; Constructors
               ;; make-string string         <= in the core
               string-tabulate

               ;; List/string conversion
               string->list
               ;; list->string               <= in the core
               reverse-list->string
               string-join

               ;; Selection
               ;; string-length string-ref   <= in the core
               string-copy
               substring/shared
               string-copy!
               string-take string-take-right
               string-drop string-drop-right
               string-pad string-pad-right
               string-trim string-trim-right
               string-trim-both

               ;; Modification
               ;; string-set!                <= in the core
               string-fill!

               ;; Comparison
               string-compare string-compare-ci
               string= string<>
               string< string>
               string<= string>=
               string-ci= string-ci<>
               string-ci< string-ci>
               string-ci<= string-ci>=
               ;;string-hash string-hash-ci   ;; (exported -- see below)

               ;; Prefixes/Suffixes
               string-prefix-length
               string-prefix-length-ci
               string-suffix-length
               string-suffix-length-ci
               string-prefix?
               string-prefix-ci?
               string-suffix?
               string-suffix-ci?

               ;; Searching
               string-index string-index-right
               string-skip string-skip-right
               string-count
               string-contains string-contains-ci

               ;; Alphabetic case mapping
               string-upcase string-upcase!
               string-downcase string-downcase!
               string-titlecase string-titlecase!

               ;; Reverse/Append
               string-reverse string-reverse!
               ;; string-append                    <= in the core
               string-append/shared
               string-concatenate
               string-concatenate-reverse
               string-concatenate/shared
               string-concatenate-reverse/shared

               ;; Fold/Unfold/Map
               string-map string-map!
               string-fold
               string-fold-right
               string-unfold
               string-unfold-right
               string-for-each
               string-for-each-index

               ;; Replicate/Rotate
               xsubstring string-xcopy!

               ;; Miscellaneous
               string-replace
               string-tokenize

               ;; Filtering/Deleting
               string-filter
               string-delete)
  #:export (string-hash string-hash-ci))

(cond-expand-provide #t '(srfi-13))

;; Return a hash value of the string @var{s} in the range 0 @dots{}
;; @var{bound} - 1.  @code{string-hash-ci} is the case-insensitive variant.
;; Optional third and fourth args @var{start} and @var{end} delimit the string.
;;
;;-sig: (s [bound [start [end]]])
;;
(define (string-hash s . rest)
  (let ((bound (if (pair? rest)
                   (or (car rest)
                       871)
                   871))
        (start (if (and (pair? rest) (pair? (cdr rest)))
                   (cadr rest)
                   0))
        (end (if (and (pair? rest) (pair? (cdr rest)) (pair? (cddr rest)))
                 (caddr rest)
                 (string-length s))))
    (hash (substring/shared s start end) bound)))

;; Return a hash value of the string @var{s} in the range 0 @dots{}
;; @var{bound} - 1.  This is the case-insensitive variant of
;; @code{string-hash}.  Optional third and fourth args @var{start}
;; and @var{end} delimit the string.
;;
;;-sig: (s [bound [start [end]]])
;;
(define (string-hash-ci s . rest)
  (let ((bound (if (pair? rest)
                   (or (car rest)
                       871)
                   871))
        (start (if (and (pair? rest) (pair? (cdr rest)))
                   (cadr rest)
                   0))
        (end (if (and (pair? rest) (pair? (cdr rest)) (pair? (cddr rest)))
                 (caddr rest)
                 (string-length s))))
    (hash (string-upcase (substring/shared s start end)) bound)))

;;; srfi-13.scm ends here
