;;; srfi-26.scm --- Curry which is not Curry

;; Copyright (C) 2003, 2004, 2007 Thien-Thi Nguyen
;; Copyright (C) 2002 Free Software Foundation, Inc.
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Author: Daniel Skarda <0rfelyus@ucw.cz>

;;; Commentary:

;;   during summer I browsed through srfi specification and as an excercice I
;; implemented srfi-26 (curry-which-is-not-curry :-). I hope somebody with
;; write access to CVS will like it and commit it.
;;
;;   I found cut and cute macros very handy (they usually safe you a lot of
;; boring typing and screen space :-) so maybe they should be also part of
;; ice-9/boot-9.scm.

;;; Code:

(define-module (srfi srfi-26)
  #:export-syntax (cut cute))

;; Expand to a procedure that specializes @code{slot} and @code{slots},
;; if they are the special symbol @code{<>} or @code{<...>}.  The latter
;; can only appear as the last element of @code{slots}.  Elements that
;; are neither of these special symbols are evaluated at run-time in
;; their respective positions.
;;
(define-macro (cut slot . slots)
  (let loop ((slots     (cons slot slots))
             (params    '())
             (args      '()))
    (if (null? slots)
        `(lambda ,(reverse! params)
           ,(reverse! args))
        (let ((s    (car slots))
              (rest (cdr slots)))
          (case s
            ((<>)
             (let ((var (gensym)))
               (loop rest (cons var params) (cons var args))))
            ((<...>)
             (and (pair? rest)
                  (error "<...> not on the end of cut expression"))
             (let ((var (gensym)))
               `(lambda ,(append! (reverse! params) var)
                  (apply ,@(reverse! (cons var args))))))
            (else
             (loop rest params (cons s args))))))))

;; Expand to a procedure that specializes @code{slots},
;; if they are the special symbol @code{<>} or @code{<...>}.  The latter
;; can only appear as the last element of @code{slots}.  Elements that
;; are neither of these special symbols are evaluated at expansion-time
;; in their respective positions.
;;
(define-macro (cute . slots)
  (let ((temp (map (lambda (s)
                     (and (not (memq s '(<> <...>)))
                          (gensym)))
                   slots)))
    `(let ,(delq! #f (map (lambda (t s)
                            (and t (list t s)))
                          temp slots))
       (cut ,@(map (lambda (t s)
                     (or t s))
                   temp slots)))))

;;; srfi-26.scm ends here
