;;; srfi-39.scm --- Parameter objects

;; Copyright (C) 2004, 2007 Thien-Thi Nguyen
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 3, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.

;;; Author: Jose Antonio Ortega Ruiz <jao@gnu.org>
;;; Date: 2004-05-05

;;; Commentary:

;; This is an implementation of SRFI-39 (Parameter objects).
;;
;; The implementation is based on Guile's fluid objects, and is, therefore,
;; thread-safe (parameters are thread-local).
;;
;; In addition to the forms defined in SRFI-39 (`make-parameter',
;; `parameterize'), a new procedure `with-parameters*' is provided.
;; This procedures is analogous to `with-fluids*' but taking as first
;; argument a list of parameter objects instead of a list of fluids.

;;; Code:

(define-module (srfi srfi-39)
  #:use-module (ice-9 syncase)
  #:export (make-parameter)
  #:export-syntax (parameterize)

  ;; helper procedure not in srfi-39.
  #:export (with-parameters*))

;; Make 'srfi-39 available as a feature identifiere to `cond-expand'.
;;
(cond-expand-provide #t '(srfi-39))

(define get-fluid-tag (lambda () 'get-fluid))
(define get-conv-tag (lambda () 'get-conv))

;; Return a new parameter object to a cell in the global dynamic environment
;; created by calling @var{converter} on @var{init}.  If @var{converter} is
;; not specified, it defaults to @code{identity}.
;;
;;-sig: (init [converter])
;;
(define (make-parameter init . converter)
  (let ((value (make-fluid))
        (conv (if (pair? converter) (car converter) identity)))
    (begin
      (fluid-set! value (conv init))
      (lambda args
        (cond
         ((null? args) (fluid-ref value))
         ((eq? (car args) get-fluid-tag) value)
         ((eq? (car args) get-conv-tag) conv)
         ((null? (cdr args)) (fluid-set! value (conv (car args))))
         (else (error "make-parameter expects 0 or 1 arguments" args)))))))

;; Dynamically bind parameter objects specified in @var{bindings}
;; and evaluate @var{body}.  Each binding in @var{bindings} has the
;; form @code{(expr1 expr2)}, where @var{expr1} evaluates to a parameter
;; object.  The order of evaluation for @var{expr1} and @var{expr2} is
;; not specified.
;;
;;-sig: (bindings [body...])
;;
(define-syntax parameterize
  (syntax-rules ()
    ((_ ((?param ?value) ...) ?body ...)
     (with-parameters* (list ?param ...)
                       (list ?value ...)
                       (lambda () ?body ...)))))

(define (with-parameters* params values thunk)
  (with-fluids* (map (lambda (p) (p get-fluid-tag)) params)
                (map (lambda (p v) ((p get-conv-tag) v)) params values)
                thunk))

;;; srfi-39.scm ends here
